local ADDON_NAME="AVR"
local L = LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME, true)

local _G = _G
local pairs = pairs
local ipairs = ipairs
local remove = table.remove
local insert = table.insert
local max = math.max
local min = math.min
local abs = math.abs
local atan2 = math.atan2
local sin = math.sin
local cos = math.cos
local type = type
local select = select
local next = next
local GetTime = GetTime
local deg2rad = math.pi/180.0
local pi = math.pi
local tostring,tonumber=tostring,tonumber
local strlen=string.len

local Core=AVR

AVRBigWigsModule=Core:NewModule("AVRBigWigsModule","AceEvent-3.0","AceTimer-3.0")

function AVRBigWigsModule:OnInitialize()
end

function AVRBigWigsModule:OnEnable()
	self:RegisterMessage("BigWigs_Message")
end

function AVRBigWigsModule:OnDisable()
	self:UnregisterMessage("BigWigs_Message")	
end

local sceneObject=nil
function AVRBigWigsModule:BigWigs_Message(...)
	if sceneObject then sceneObject:BigWigs_Message(...) end
end

local stripColor=function(s)
	return s:gsub("|cff%x%x%x%x%x%x([^|]*)|r","%1")
end

local splitTarget=function(target)
	if target==nil then return {} end
	local targets={}
	local pos=strfind(target,",")
	while pos do
		local t=target:sub(1,pos-1):gsub("^%s*",""):gsub("%s*$","")
		if strlen(t)>0 then insert(targets,t) end
		target=target:sub(pos+1)
		pos=strfind(target,",")
	end
	local t=target:gsub("^%s*",""):gsub("%s*$","")
	if strlen(t)>0 then insert(targets,t) end
	return targets
end

local onYouPattern = nil
local onOtherPattern = nil

local getTargetFromMessage=function(text)
	if not onYouPattern then
		local BWL = LibStub("AceLocale-3.0"):GetLocale("Big Wigs: Common")
		if not BWL then return nil end
		onYouPattern = BWL["you"]:gsub("%%s","(.+)")
		onOtherPattern = BWL["other"]:gsub("%%s","(.+)")
	end
	
	local target=nil
	if text:match(onYouPattern) then
		return "player"
	else
		local _,target=text:match(onOtherPattern)
		if target then return stripColor(target)
		else return nil end
	end
end

AVRBigWigsScene={Embed=Core.Embed}

AVRBigWigsScene.class="AVRBigWigsScene"
function AVRBigWigsScene:New(threed)
	if self ~= AVRBigWigsScene then return end
	
	if sceneObject ~=nil then
		Core:Print(L["You can add only one BigWigs scene"])
		return nil
	end
	
	local s=AVRScene:New(threed)
	AVRBigWigsScene:Embed(s)
	s.class=AVRBigWigsScene.class
	s.autoFireOptionChanges=false
	s.name=L["BigWigs scene"]
	s.options={
		IcecrownCitadel={
			malleableGoo=true,
			defile=true,
			frostBeacon=true,
			gasSpores=true,
			unstableOozeExplosion=true
		}
	}
	sceneObject=s
	
	return s
end

function AVRBigWigsScene:OnRemove()
	sceneObject=nil
end

function AVRBigWigsScene:MarkRaid(follow,radius,duration,r,g,b,a,segments)
	segments=segments or 8
	local raidSize=GetNumRaidMembers()
	for i=1,raidSize do
		local _,_,subGroup,_,_,_,_,_,dead=GetRaidRosterInfo(i)
		if subGroup and subGroup<=5 and not dead then
			self:AddCircleWarning("raid"..i,follow,radius,duration,r,g,b,a,segments)
		end
	end
end

function AVRBigWigsScene:AddCircleWarning(unit,follow,radius,duration,r,g,b,a,segments)
	if follow==nil then follow=true end
	radius=radius or 5.0
	duration=duration or 5.0
	r=r or 1.0
	g=g or 0.0
	b=b or 0.0
	a=a or 0.5
	
	local px,py
	if not follow then
		px,py=self.threed:GetUnitPosition(unit)
		if px==0 and py==0 then return end
	end
	local m=AVRFilledCircleMesh:New(radius,segments)
	m:SetColor(r,g,b,a)
	if follow then
		m.followUnit=unit
	else
		m:TranslateMesh(px,py)
	end
	self:AddMesh(m)
	AVRBigWigsModule:ScheduleTimer(function()
		m:Remove()
	end,duration)
end

function AVRBigWigsScene:BigWigs_Message(message, module, key, text, color, noraidsay, sound, broadcastOnly, icon)
	if key==72295 then 
		if not self.options.IcecrownCitadel.malleableGoo then return end
		if module:GetName()=="Festergut" then
			-- this is also triggered on heroic festergut but we can't get target for it so just ignore it
		else
			if broadcastOnly then return end
			local target=getTargetFromMessage(text)
			
			if target then
				self:AddCircleWarning(target,false,5,5)
			end
		end
	elseif key==72762 then -- doesn't work because Lich King room is bugged
		if not self.options.IcecrownCitadel.defile then return end
		if broadcastOnly then return end
		local target=getTargetFromMessage(text)
		
		if target then
			self:AddCircleWarning(target,true,5,2)
		end
	elseif key==70126 then 
		if not self.options.IcecrownCitadel.frostBeacon then return end
		local target=getTargetFromMessage(text)
		local targets=splitTarget(target)
		for i,t in ipairs(targets) do
			self:AddCircleWarning(t,true,5,7)
		end
	elseif key==69279 then
		if not self.options.IcecrownCitadel.gasSpores then return end
		local target=getTargetFromMessage(text)
		local targets=splitTarget(target)
		for i,t in ipairs(targets) do
			self:AddCircleWarning(t,true,8,12)
		end
	elseif key==69839 then
		if not self.options.IcecrownCitadel.unstableOozeExplosion then return end
		self:MarkRaid(false,6,10,nil,nil,nil,0.2,10)
	end
end

function AVRBigWigsScene:GetOptions()
	return {
		type = "group",
		name = self.name,	
		get = function(info) 
			if type(info)~="string" then info=info[#info] end
			local func="Get"..string.upper(string.sub(info,1,1))..string.sub(info,2)
			if self[func] then return self[func](self) 
			else return self[info] end
		end,
		set = function(info,val)
			if type(info)~="string" then info=info[#info] end
			local func="Set"..string.upper(string.sub(info,1,1))..string.sub(info,2)
			if self[func] then self[func](self,val)
			else self[info]=val end
		end,
		args = {
			name = {
				type = "input",
				name = L["Name"],
				order = 10,
				width = "full",
				set = 	function(_,val)
							self.name=val
							Core:OptionsChanged()
						end
			},
			visible = {
				type = "toggle",
				name = L["Visible"],
				order = 15
			},
			select = {
				type = "toggle",
				name = L["Select"],
				order = 20,
				get =	function(_)
							if Core.sceneManager:GetSelectedScene()==self then return true
							else return false end
						end,
				set =	function(_,val)
							if val then Core.sceneManager:SetSelectedScene(self)
							else Core.sceneManager:SetSelectedScene(nil) end
						end
			},
			remove = {
				type = "execute",
				name = L["Remove scene"],
				order = 30,
				width = "full",
				confirm = true,
				func = function() self:Remove() end
			},
			options = {
				type = "group",
				name = L["Boss options"],
				childGroups = "select",
				order = 40,
				get = 	function(info)
							return self.options[info[#info-1]][info[#info]]
						end,
				set =	function(info,val)
							self.options[info[#info-1]][info[#info]]=val
						end,
				args = {
					IcecrownCitadel = {
						name = L["Icecrown Citadel"],
						type = "group",
						args = {
							malleableGoo={
								name=L["Malleable goo"],
								type="toggle"
							},
							--[[defile={
								name=L["Defile"],
								type="toggle"
							},]]
							frostBeacon={
								name=L["Frost beacon"],
								type="toggle"
							},
							unstableOozeExplosion={
								name=L["Unstable ooze explosion"],
								type="toggle"
							},
							gasSpores={
								name=L["Gas spores"],
								type="toggle"
							},
						}
					}
				}
			}
		}
	}
end

local function copyTable(t,r)
	if not r then r={} end
	for k,v in pairs(t) do
		if type(v)=="table" then
			r[k]=copyTable(v)
		else
			r[k]=v
		end
	end
	return r
end

function AVRBigWigsScene:Pack()
	local temp=self.meshes
	self.meshes={}
	local s=AVRScene.Pack(self)
	s.options=copyTable(self.options)

	self.meshes=temp
	return s
end

function AVRBigWigsScene:Unpack(s)
	AVRScene.Unpack(self,s)
	if s.options then
		copyTable(s.options,self.options)
	end

end

AVR:RegisterSceneClass(AVRBigWigsScene,L["BigWigs scene"],false)
